/* PLMAIN.C : Main module of PICLAB program, in ANSI C.  Details of system
** can be found in PICLAB.DOC.	In this module are a few globally defined
** utility routines and main().  The only command handler is RUN.
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>
#include <string.h>
#include <io.h>

#define MAIN_MODULE
#include "piclab.h"
#include "yiq"			/* These last four are not needed in this module,	*/
#include "sierra"		/* but are included with MAIN_MODULE defined so 	*/
#include "pjtbls"		/* they will be initialized.						*/
#include "hash"

char *signon[] = {
	" ",
	"#######   ##   ######   ##         ######   #######     Version 1.10",
	"##    ##  ##  ##    ##  ##        ##    ##  ##    ##    July 6, 1989",
	"##    ##  ##  ##        ##        ##    ##  ##    ##  by",
	"#######   ##  ##        ##        ########  #######     Lee Daniel Crocker",
	"##        ##  ##        ##        ##    ##  ##    ##",
	"##        ##  ##    ##  ##        ##    ##  ##    ##    CIS ID 73407,2030",
	"##        ##   ######   ########  ##    ##  #######     BIX ID lcrocker",
	" ",
	"########################## ### ## # # # #  # #  #",
	"########## #### ## # # # ##  # # ## ## # ## # ##  ## ## # # # # #",
	"######## #### ### ##### # ##### # # # # #        #       #        #  #",
	"############ ## ### # ## ## #  # # # # # ###### #  # # ##   #  #",
	"###### ########### ### ### # ## ### # # #      #  #   #    #  #    #     #",
	"########## #### # ## ### #### ##   # # # ### #   #  #    #   #   #",
	" ",
	"This is a public domain product which may be distributed freely and used",
	"without restrictions of any kind.  It should be distributed along with a",
	"text file named PICLAB.DOC which explains it in more detail.  If you do not",
	"have this document, or if you have other questions about PICLAB, you can",
	"contact the author via CompuServe Information Service or Byte Information",
	"Exchange.  Type LIST COMMANDS for a list of currently valid commands.",
	NULL };

static int dorun(char *);

void pl_printf(char *fmt, ...)
{
    va_list args;

    if (!interactive) return;
    va_start(args, fmt);
    vprintf(fmt, args);
    va_end(args);
}

static char *warnings[] = { "",
	"Command arguments ignored",
	"Open files were closed",
	"Probable information loss",
	"" };

static char *errors[] = { "",
	"Assertion failure; contact author",
	"Insufficient memory",
	"Miscellaneous file I/O",
	"Unexpected end of file",
	"LZW compression/decompression",
	"Unrecognized file format or bad file",
	"Image buffer empty",
	"Point transform pending; issue TRANSFORM command",
	"Illegal parameter values",
	"File not found",
	"" };

void pl_warn(int msg)
{
	if (msg < 1 || msg > 3) pl_printf("** W %d **\n", msg);
	else pl_printf("Warning: %s.\n", warnings[msg]);
}

void pl_trace(register int line)
{
	if ((line & 7) == 7) pl_printf("\rLine%5d", line+1);
}

void pl_init()
{
    char *cp;
	int i;

    interactive = isatty(fileno(stdin));
	if ((cp = getenv("TMP")) != NULL) strcpy(tempdir, cp);
	else *tempdir = '\0';
	strupr(tempdir);

	if ((cp = getenv("PRINTER")) != NULL) strcpy(printer, cp);
	strupr(printer);

	for (i=0; i<256; ++i) {
		lookup[0][i] = lookup[1][i] = lookup[2][i] = (U8)i;
	}
	old = &images[0];	old->bufname = "OLD";	old->filename = "PLTEMP1";
	new = &images[1];	new->bufname = "NEW";	new->filename = "PLTEMP2";
	itmp = &images[2];	itmp->bufname = "TEMP"; itmp->filename = "PLTEMP3";

	old->planes = new->planes = itmp->planes =
	old->flags = new->flags = itmp->flags = 0;
	old->width = new->width = itmp->width =
	old->height = new->height = itmp->height = 0;
}

void pl_cleanup()
{
	if (!leavetemps) {
	}
}

/* The parse() function will be the main interpreter loop of PICLAB.  It
** will accept input a line at a time, parse it into a sequence of words,
** and call the function named by the first word (in cmdtable[]) with
** the second and subsequent words as arguments.
*/

int pl_match(char *key, char *words[])
{
    int i, l;

    l = strlen(key);
    for (i=0; words[i] != NULL; ++i) {
        if (strncmp(key, words[i], l) == 0) return i+1;
    }
    return 0;
}

int pl_parse(char *inline)
{
	int argc;
	argument argv[10];
    char *cp, *word;
	int i, l, e;

    argc = 0;
    cp = strupr(inline);
    while (*cp) {
        while (*cp && *cp <= ' ') ++cp;
        word = cp;
        if (!*cp) break;
        while (*cp > ' ') ++cp;
        *cp++ = '\0';
        argv[argc].cval = word;
        argv[argc].fval = (float)atof(word);
        ++argc;
    }
	if (argc == 0) return 0;
    word = argv[0].cval;
    l = strlen(word);
    for (i=0; cmdtable[i].command != NULL; ++i) {
        if (strncmp(word, cmdtable[i].command, l) == 0) {
			e = (*cmdtable[i].function)(argc, argv);
			releaseall();
			if (e != 0) {
				if (e < 1 || e > 10) pl_printf("** E %d **\n", e);
				else pl_printf("Error: %s.\n", errors[e]);
				if (p_closeall()) pl_warn(2);
			}
			return e;
        }
    }
	pl_printf("Unrecognized command.\n");
}

static int dorun(char *progname)
{
	U32 mem;
	char inline[128];
	int r;
	FILE *pfile;

	if ((pfile = fopen(progname, "r")) == NULL) return 3;
	mem = mark();
	do {
		release(mem);
		if (fgets(inline, 128, pfile) == NULL) break;
		r = pl_parse(inline);
	} while (r == 0);
	release(mem);
	fclose(pfile);
	return r;
}

int run(int ac, argument *av)
{
	int r, intflag;

	intflag = interactive;
	if (ac < 2) return 9;
	if (ac > 2 && *av[2].cval == 'E') {
		if (ac > 3) pl_warn(1);
		interactive = 1;
	} else {
		interactive = 0;
	}
	r = dorun(av[1].cval);
	interactive = intflag;
	return r;
}

void main(int argc, char *argv[])
{
	int i, *ip, junk, c, r; 	/* Do not declare register variables here	*/
	char inline[128];			/* because setjmp() is being used.			*/

    pl_init();

    ip = &junk;
    while (--argc) {
        if (**++argv == '-') {
            while (c = *++*argv) {
                if (isdigit(c)) {
                    *ip *= 10;
                    *ip += (c - '0');
                } else switch (toupper(c)) {
                    case 'I':   interactive = 1;    *ip = 0;    break;
					case 'L':	leavetemps = 1; 	*ip = 0;	break;
                }
            }
        } else {
			if (dorun(*argv) != 0) pl_printf("Program \"%s\" returned error.\n", *argv);
        }
    }

	if (!quitflag) {
		for (i=0; signon[i]; ++i) pl_printf("%s\n", signon[i]);

		if (setjmp(interp) != 0) {
			releaseall();
			if (p_closeall()) pl_printf("\nTemp files were closed;");
			pl_printf("\nError recovery complete.\n");
		}
		do {
			if (interactive) pl_printf("\nPL>");
			if (fgets(inline, 128, stdin) == NULL) quitflag = 1;
			else r = pl_parse(inline);
		} while (!quitflag);
	}

    pl_cleanup();
}
